@@ -90,8 +90,9 @@ residuals_phregr <- function(
   
   elements = c(stratum, time, event, covariates, weight, offset, id)
   elements = unique(elements[elements != "" & elements != "none"])
-  mf = model.frame(formula(paste("~", paste(elements, collapse = "+"))),
-                   data = data)
+  fml = formula(paste("~", paste(elements, collapse = "+")))
+  mf = model.frame(fml, data = data, na.action = na.omit)
+  
   rownum = as.integer(rownames(mf))
   df = data[rownum,]
   
@@ -100,15 +101,13 @@ residuals_phregr <- function(
     covariates[1] == "" || tolower(covariates[1]) == "none"))) {
     p3 = 0
   } else {
-    t1 = terms(formula(paste("~", paste(covariates, collapse = "+"))))
-    t2 = attr(t1, "factors")
-    t3 = rownames(t2)
-    p3 = length(t3)
+    fml1 = formula(paste("~", paste(covariates, collapse = "+")))
+    p3 = length(rownames(attr(terms(fml1), "factors")))
   }
 
   if (p >= 1 && p3 >= 1) {
-    mf = model.frame(t1, df)
-    mm = model.matrix(t1, mf)
+    mf1 <- model.frame(fml1, data = df, na.action = na.pass)
+    mm <- model.matrix(fml1, mf1)
     colnames(mm) = make.names(colnames(mm))
     varnames = colnames(mm)[-1]
     for (i in 1:length(varnames)) {
