@@ -130,6 +130,8 @@
 #'   \code{"d_star"}, \code{"treated"}, \code{base_cov}, and \code{treat}.
 #'
 #' * \code{fit_outcome}: The fitted outcome Cox model.
+#' 
+#' * \code{fail}: Whether a model fails to converge.
 #'
 #' * \code{settings}: A list with the following components:
 #'
@@ -169,6 +171,9 @@
 #' * \code{psi_trt_CI}: The confidence interval for \code{psi_trt} if
 #'   \code{swtrt_control_only} is \code{FALSE}.
 #'
+#' * \code{fail_boots}: The indicators for failed bootstrap samples
+#'   if \code{boot} is \code{TRUE}.
+#'
 #' * \code{hr_boots}: The bootstrap hazard ratio estimates if \code{boot} is
 #'   \code{TRUE}.
 #'
@@ -243,8 +248,8 @@ tsesimp <- function(data, id = "id", stratum = "", time = "time",
 
   elements = c(stratum, time, event, treat, censor_time, pd, swtrt)
   elements = unique(elements[elements != "" & elements != "none"])
-  mf = model.frame(formula(paste("~", paste(elements, collapse = "+"))),
-                   data = data)
+  fml = formula(paste("~", paste(elements, collapse = "+")))
+  mf = model.frame(fml, data = data, na.action = na.omit)
 
   rownum = as.integer(rownames(mf))
   df = data[rownum,]
@@ -254,14 +259,13 @@ tsesimp <- function(data, id = "id", stratum = "", time = "time",
     base_cov[1] == "" || tolower(base_cov[1]) == "none"))) {
     p = 0
   } else {
-    t1 = terms(formula(paste("~", paste(base_cov, collapse = "+"))))
-    t2 = attr(t1, "factors")
-    t3 = rownames(t2)
-    p = length(t3)
+    fml1 = formula(paste("~", paste(base_cov, collapse = "+")))
+    p = length(rownames(attr(terms(fml1), "factors")))
   }
 
   if (p >= 1) {
-    mm = model.matrix(t1, df)
+    mf1 <- model.frame(fml1, data = df, na.action = na.pass)
+    mm <- model.matrix(fml1, mf1)
     colnames(mm) = make.names(colnames(mm))
     varnames = colnames(mm)[-1]
     for (i in 1:length(varnames)) {
@@ -278,14 +282,13 @@ tsesimp <- function(data, id = "id", stratum = "", time = "time",
     base2_cov[1] == "" || tolower(base2_cov[1]) == "none"))) {
     p2 = 0
   } else {
-    t1 = terms(formula(paste("~", paste(base2_cov, collapse = "+"))))
-    t2 = attr(t1, "factors")
-    t3 = rownames(t2)
-    p2 = length(t3)
+    fml2 = formula(paste("~", paste(base2_cov, collapse = "+")))
+    p2 = length(rownames(attr(terms(fml2), "factors")))
   }
 
   if (p2 >= 1) {
-    mm2 = model.matrix(t1, df)
+    mf2 <- model.frame(fml2, data = df, na.action = na.pass)
+    mm2 = model.matrix(fml2, mf2)
     colnames(mm2) = make.names(colnames(mm2))
     varnames2 = colnames(mm2)[-1]
     for (i in 1:length(varnames2)) {
