@@ -409,6 +409,8 @@ List ipcwcpp(
                 NumericMatrix& zb, NumericMatrix& zb_cox_den,
                 NumericMatrix& zb_lgs_den)->List {
                   int h, i, j, n = static_cast<int>(idb.size());
+                  bool fail = 0; // whether any model fails to converge
+                  NumericVector init(1, NA_REAL);
                   
                   // order data by treat, id, and time
                   IntegerVector order = seq(0, n-1);
@@ -525,7 +527,7 @@ List ipcwcpp(
                     if (!swtrt_control_only) {
                       DataFrame a = survsplit(tstart1, tstop1, cut);
                       IntegerVector censor = a["censor"];
-                      l = a["row"];
+                      IntegerVector l = a["row"];
                       id2 = id1[l];
                       stratum2 = stratum1[l];
                       tstart2 = a["start"];
@@ -544,7 +546,7 @@ List ipcwcpp(
                       }
                     } else {
                       // extract data for the control group
-                      l = which(treat1 == 0);
+                      IntegerVector l = which(treat1 == 0);
                       IntegerVector id0 = id1[l];
                       IntegerVector stratum0 = stratum1[l];
                       NumericVector tstart0 = tstart1[l];
@@ -610,7 +612,7 @@ List ipcwcpp(
                     
                     // fit the switching models by treatment group
                     for (h=0; h<K; h++) {
-                      l = which(treat2 == h);
+                      IntegerVector l = which(treat2 == h);
                       IntegerVector id3 = id2[l];
                       IntegerVector stratum3 = stratum2[l];
                       NumericVector tstart3 = tstart2[l];
@@ -636,10 +638,14 @@ List ipcwcpp(
                       // fit the denominator model for crossover
                       List fit_den = phregcpp(
                         data1, "", "ustratum", "tstart", "tstop", "cross",
-                        denominator, "", "", "uid", ties, 
+                        denominator, "", "", "uid", ties, init, 
                         1, 1, 0, 0, 0, alpha, 50, 1.0e-9);
                       
                       // obtain the survival probabilities for crossover
+                      DataFrame sumstat_den = DataFrame(fit_den["sumstat"]);
+                      bool fail_den = sumstat_den["fail"];
+                      if (fail_den == 1) fail = 1;
+                      
                       DataFrame parest_den = DataFrame(fit_den["parest"]);
                       NumericVector beta_den = parest_den["beta"];
                       NumericMatrix vbeta_den(p2, p2);
@@ -655,11 +661,15 @@ List ipcwcpp(
                       
                       List fit_num = phregcpp(
                         data1, "", "ustratum", "tstart", "tstop", "cross", 
-                        numerator, "", "", "uid", ties, 
+                        numerator, "", "", "uid", ties, init, 
                         1, 1, 0, 0, 0, alpha, 50, 1.0e-9);
                       
                       NumericVector surv_num(m);
                       if (p1 > 0) {
+                        DataFrame sumstat_num = DataFrame(fit_num["sumstat"]);
+                        bool fail_num = sumstat_num["fail"];
+                        if (fail_num == 1) fail = 1;
+                        
                         DataFrame parest_num = DataFrame(fit_num["parest"]);
                         NumericVector beta_num = parest_num["beta"];
                         NumericMatrix vbeta_num(p1, p1);
@@ -824,7 +834,7 @@ List ipcwcpp(
                     
                     // fit the switching models by treatment group
                     for (h=0; h<K; h++) {
-                      l = which(treat1 == h);
+                      IntegerVector l = which(treat1 == h);
                       IntegerVector id2 = id1[l];
                       IntegerVector stratum2 = stratum1[l];
                       NumericVector tstart2 = tstart1[l];
@@ -868,39 +878,32 @@ List ipcwcpp(
                       
                       List fit_den = logisregcpp(
                         data1, "", "cross", covariates_lgs_den, "", "", 
-                        "", "uid", "logit", 0, firth, flic, 0, alpha,
-                        50, 1.0e-9);
+                        "", "uid", "logit", init, 
+                        0, firth, flic, 0, alpha, 50, 1.0e-9);
+                      
+                      DataFrame sumstat_den = DataFrame(fit_den["sumstat"]);
+                      bool fail_den = sumstat_den["fail"];
+                      if (fail_den == 1) fail = 1;
                       
                       DataFrame f_den = DataFrame(fit_den["fitted"]);
                       NumericVector h_den = f_den["fitted_values"];
                       
                       // convert to probability of remaining uncensored
-                      NumericVector s_den = 1.0 - h_den;
-                      
-                      // replace missing probabilities with 1 within subjects
-                      l = which(treat1 == h);
-                      IntegerVector id3 = id1[l];
-                      NumericVector tstop3 = tstop1[l];
-                      int n3 = static_cast<int>(l.size());
-                      
-                      // match on id and time
-                      IntegerVector sub = match3(id2, tstop2, id3, tstop3);
-                      NumericVector pstay_den(n3, 1.0);
-                      pstay_den[sub] = s_den;
+                      NumericVector pstay_den = 1.0 - h_den;
                       
                       // obtain cumulative products within a subject
                       IntegerVector idx(1,0);
-                      for (i=1; i<n3; i++) {
-                        if (id3[i] != id3[i-1]) {
+                      for (i=1; i<n2; i++) {
+                        if (id2[i] != id2[i-1]) {
                           idx.push_back(i);
                         }
                       }
                       
-                      int nids3 = static_cast<int>(idx.size());
-                      idx.push_back(n3);
+                      int nids2 = static_cast<int>(idx.size());
+                      idx.push_back(n2);
                       
-                      NumericVector surv_den(n3);
-                      for (i=0; i<nids3; i++) {
+                      NumericVector surv_den(n2);
+                      for (i=0; i<nids2; i++) {
                         surv_den[idx[i]] = pstay_den[idx[i]];
                         for (j=idx[i]+1; j<idx[i+1]; j++) {
                           surv_den[j] = surv_den[j-1]*pstay_den[j];
@@ -909,19 +912,21 @@ List ipcwcpp(
                       
                       List fit_num = logisregcpp(
                         data1, "", "cross", covariates_lgs_num, "", "", 
-                        "", "uid", "logit", 0, firth, flic, 0, alpha, 
-                        50, 1.0e-9);
+                        "", "uid", "logit", init, 
+                        0, firth, flic, 0, alpha, 50, 1.0e-9);
+                      
+                      DataFrame sumstat_num = DataFrame(fit_num["sumstat"]);
+                      bool fail_num = sumstat_num["fail"];
+                      if (fail_num == 1) fail = 1;
                       
                       DataFrame f_num = DataFrame(fit_num["fitted"]);
                       NumericVector h_num = f_num["fitted_values"];
                       
                       // convert to probability of remaining uncensored
-                      NumericVector s_num = 1.0 - h_num;
-                      NumericVector pstay_num(n3, 1.0);
-                      pstay_num[sub] = s_num;
-                      
-                      NumericVector surv_num(n3);
-                      for (i=0; i<nids3; i++) {
+                      NumericVector pstay_num = 1.0 - h_num;
+
+                      NumericVector surv_num(n2);
+                      for (i=0; i<nids2; i++) {
                         surv_num[idx[i]] = pstay_num[idx[i]];
                         for (j=idx[i]+1; j<idx[i+1]; j++) {
                           surv_num[j] = surv_num[j-1]*pstay_num[j];
@@ -937,13 +942,13 @@ List ipcwcpp(
                         // truncated unstabilized weights
                         if (trunc_upper_only) {
                           double upper = quantilecpp(w, 1-trunc);
-                          for (i=0; i<n3; i++) {
+                          for (i=0; i<n2; i++) {
                             if (w[i] > upper) w[i] = upper;
                           }
                         } else {
                           double lower = quantilecpp(w, trunc);
                           double upper = quantilecpp(w, 1-trunc);
-                          for (i=0; i<n3; i++) {
+                          for (i=0; i<n2; i++) {
                             if (w[i] < lower) {
                               w[i] = lower;
                             } else if (w[i] > upper) {
@@ -955,13 +960,13 @@ List ipcwcpp(
                         // truncated stabilized weights
                         if (trunc_upper_only) {
                           double upper = quantilecpp(sw, 1-trunc);
-                          for (i=0; i<n3; i++) {
+                          for (i=0; i<n2; i++) {
                             if (sw[i] > upper) sw[i] = upper;
                           }
                         } else {
                           double lower = quantilecpp(sw, trunc);
                           double upper = quantilecpp(sw, 1-trunc);
-                          for (i=0; i<n3; i++) {
+                          for (i=0; i<n2; i++) {
                             if (sw[i] < lower) {
                               sw[i] = lower;
                             } else if (sw[i] > upper) {
@@ -1038,14 +1043,18 @@ List ipcwcpp(
                     fit_outcome = phregcpp(
                       data_outcome, "", "ustratum", "tstart", "tstop",
                       "event", covariates, "stabilized_weight", "",
-                      "uid", ties, 1, 0, 0, 0, 0, alpha, 50, 1.0e-9);
+                      "uid", ties, init, 1, 0, 0, 0, 0, alpha, 50, 1.0e-9);
                   } else {
                     fit_outcome = phregcpp(
                       data_outcome, "", "ustratum", "tstart", "tstop",
                       "event", covariates, "unstabilized_weight", "",
-                      "uid", ties, 1, 0, 0, 0, 0, alpha, 50, 1.0e-9);
+                      "uid", ties, init, 1, 0, 0, 0, 0, alpha, 50, 1.0e-9);
                   }
                   
+                  DataFrame sumstat_cox = DataFrame(fit_outcome["sumstat"]);
+                  bool fail_cox = sumstat_cox["fail"];
+                  if (fail_cox == 1) fail = 1;
+                  
                   DataFrame parest = DataFrame(fit_outcome["parest"]);
                   NumericVector beta = parest["beta"];
                   NumericVector sebeta = parest["sebeta"];
@@ -1065,13 +1074,15 @@ List ipcwcpp(
                       Named("hrhat") = hrhat,
                       Named("hrlower") = hrlower,
                       Named("hrupper") = hrupper,
-                      Named("pvalue") = pvalue);
+                      Named("pvalue") = pvalue,
+                      Named("fail") = fail);
                   } else {
                     out = List::create(
                       Named("hrhat") = hrhat,
                       Named("hrlower") = hrlower,
                       Named("hrupper") = hrupper,
-                      Named("pvalue") = pvalue);
+                      Named("pvalue") = pvalue,
+                      Named("fail") = fail);
                   }
                   
                   return out;
@@ -1125,10 +1136,12 @@ List ipcwcpp(
   double hrlower = out["hrlower"];
   double hrupper = out["hrupper"];
   double pvalue = out["pvalue"];
+  bool fail = out["fail"];
   
   // construct the confidence interval for HR
   String hr_CI_type;
   NumericVector hrhats(n_boot);
+  LogicalVector fails(n_boot);
   if (!boot) { // use Cox model to construct CI for HR if no boot
     hr_CI_type = "Cox model";
   } else { // bootstrap the entire process to construct CI for HR
@@ -1189,18 +1202,21 @@ List ipcwcpp(
       out = f(idb, stratumb, tstartb, tstopb, eventb, treatb,
               swtrtb, swtrt_timeb, zb, zb_cox_den, zb_lgs_den);
       
+      fails[k] = out["fail"];
       hrhats[k] = out["hrhat"];
     }
     
     // obtain bootstrap confidence interval for HR
     double loghr = log(hrhat);
-    NumericVector loghrs = log(hrhats);
+    LogicalVector ok = 1 - fails;
+    int n_ok = sum(ok);
+    NumericVector loghrs = log(hrhats[ok]);
     double sdloghr = sd(loghrs);
-    double tcrit = R::qt(1-alpha/2, n_boot-1, 1, 0);
+    double tcrit = R::qt(1-alpha/2, n_ok-1, 1, 0);
     hrlower = exp(loghr - tcrit*sdloghr);
     hrupper = exp(loghr + tcrit*sdloghr);
     hr_CI_type = "bootstrap";
-    pvalue = 2*(1 - R::pt(fabs(loghr/sdloghr), n_boot-1, 1, 0));
+    pvalue = 2*(1 - R::pt(fabs(loghr/sdloghr), n_ok-1, 1, 0));
   }
   
   List settings = List::create(
@@ -1229,9 +1245,13 @@ List ipcwcpp(
     Named("fit_switch") = fit_switch,
     Named("data_outcome") = data_outcome,
     Named("fit_outcome") = fit_outcome,
+    Named("fail") = fail,
     Named("settings") = settings);
   
-  if (boot) result.push_back(hrhats, "hr_boots");
+  if (boot) {
+    result.push_back(fails, "fail_boots");
+    result.push_back(hrhats, "hr_boots"); 
+  }
   
   return result;
 }
