@@ -10,8 +10,7 @@ void set_seed(int seed);
 
 IntegerVector which(const LogicalVector& vector);
 
-IntegerVector findInterval3(NumericVector x,
-                            NumericVector breaks);
+IntegerVector findInterval3(NumericVector x, NumericVector v);
 
 double brent(const std::function<double(double)>& f,
              double x1, double x2, double tol);
